@@ -40,8 +40,9 @@
 #include "config.h"
 #endif
 
+#include "clutter-backend-private.h"
 #include "clutter-debug.h"
-#include "clutter-device-manager.h"
+#include "clutter-device-manager-private.h"
 #include "clutter-enum-types.h"
 #include "clutter-marshal.h"
 #include "clutter-private.h"
@@ -58,9 +59,13 @@ enum
 {
   PROP_0,
 
-  PROP_BACKEND
+  PROP_BACKEND,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 enum
 {
   DEVICE_ADDED,
@@ -117,20 +122,21 @@ static void
 clutter_device_manager_class_init (ClutterDeviceManagerClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-  GParamSpec *pspec;
 
   g_type_class_add_private (klass, sizeof (ClutterDeviceManagerPrivate));
 
+  obj_props[PROP_BACKEND] =
+    g_param_spec_object ("backend",
+                         P_("Backend"),
+                         P_("The ClutterBackend of the device manager"),
+                         CLUTTER_TYPE_BACKEND,
+                         CLUTTER_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+
   gobject_class->set_property = clutter_device_manager_set_property;
   gobject_class->get_property = clutter_device_manager_get_property;
-
-  pspec = g_param_spec_object ("backend",
-                               P_("Backend"),
-                               P_("The ClutterBackend of the device manager"),
-                               CLUTTER_TYPE_BACKEND,
-                               CLUTTER_PARAM_READWRITE |
-                               G_PARAM_CONSTRUCT_ONLY);
-  g_object_class_install_property (gobject_class, PROP_BACKEND, pspec);
+  _clutter_object_class_install_properties (gobject_class,
+                                            PROP_LAST,
+                                            obj_props);
 
   /**
    * ClutterDeviceManager::device-added:
@@ -207,7 +213,7 @@ clutter_device_manager_get_default (void)
  *
  * Lists all currently registered input devices
  *
- * Return value: (transfer container) (element-type ClutterInputDevice):
+ * Return value: (transfer container) (element-type Clutter.InputDevice):
  *   a newly allocated list of #ClutterInputDevice objects. Use
  *   g_slist_free() to deallocate it when done
  *
@@ -231,7 +237,7 @@ clutter_device_manager_list_devices (ClutterDeviceManager *device_manager)
  *
  * Lists all currently registered input devices
  *
- * Return value: (transfer none) (element-type ClutterInputDevice):
+ * Return value: (transfer none) (element-type Clutter.InputDevice):
  *   a pointer to the internal list of #ClutterInputDevice objects. The
  *   returned list is owned by the #ClutterDeviceManager and should never
  *   be modified or freed
@@ -302,6 +308,27 @@ clutter_device_manager_get_core_device (ClutterDeviceManager   *device_manager,
   return manager_class->get_core_device (device_manager, device_type);
 }
 
+void
+_clutter_device_manager_select_stage_events (ClutterDeviceManager *device_manager,
+                                             ClutterStage         *stage,
+                                             gint                  event_flags)
+{
+  ClutterDeviceManagerClass *manager_class;
+  const GSList *devices, *d;
+
+  g_return_if_fail (CLUTTER_IS_DEVICE_MANAGER (device_manager));
+
+  manager_class = CLUTTER_DEVICE_MANAGER_GET_CLASS (device_manager);
+  devices = manager_class->get_devices (device_manager);
+
+  for (d = devices; d != NULL; d = d->next)
+    {
+      ClutterInputDevice *device = d->data;
+
+      _clutter_input_device_select_stage_events (device, stage, event_flags);
+    }
+}
+
 /*
  * _clutter_device_manager_add_device:
  * @device_manager: a #ClutterDeviceManager
@@ -399,3 +426,11 @@ _clutter_device_manager_update_devices (ClutterDeviceManager *device_manager)
       _clutter_input_device_update (device);
     }
 }
+
+ClutterBackend *
+_clutter_device_manager_get_backend (ClutterDeviceManager *manager)
+{
+  g_return_val_if_fail (CLUTTER_IS_DEVICE_MANAGER (manager), NULL);
+
+  return manager->priv->backend;
+}
