@@ -320,6 +320,58 @@ static GQuark quark_layout_alpha = 0;
 
 static guint manager_signals[LAST_SIGNAL] = { 0, };
 
+static void
+layout_manager_freeze_layout_change (ClutterLayoutManager *manager)
+{
+  gpointer is_frozen;
+
+  CLUTTER_NOTE (LAYOUT, "Freezing changes for manager '%s'[%p]",
+                G_OBJECT_TYPE_NAME (manager),
+                manager);
+
+  is_frozen = g_object_get_data (G_OBJECT (manager), "freeze-change");
+  if (is_frozen == NULL)
+    g_object_set_data (G_OBJECT (manager), "freeze-change",
+                       GUINT_TO_POINTER (1));
+  else
+    {
+      guint level = GPOINTER_TO_UINT (is_frozen) + 1;
+
+      g_object_set_data (G_OBJECT (manager), "freeze-change",
+                         GUINT_TO_POINTER (level));
+    }
+}
+
+static void
+layout_manager_thaw_layout_change (ClutterLayoutManager *manager)
+{
+  gpointer is_frozen;
+
+  is_frozen = g_object_get_data (G_OBJECT (manager), "freeze-change");
+  if (is_frozen == NULL)
+    g_critical (G_STRLOC ": Mismatched thaw; you have to call "
+                "clutter_layout_manager_freeze_layout_change() prior to "
+                "calling clutter_layout_manager_thaw_layout_change()");
+  else
+    {
+      guint level = GPOINTER_TO_UINT (is_frozen);
+
+      g_assert (level > 0);
+
+      CLUTTER_NOTE (LAYOUT, "Thawing changes for manager '%s'[%p]",
+                    G_OBJECT_TYPE_NAME (manager),
+                    manager);
+
+      level -= 1;
+      if (level == 0)
+        g_object_set_data (G_OBJECT (manager), "freeze-change", NULL);
+      else
+        g_object_set_data (G_OBJECT (manager), "freeze-change",
+                           GUINT_TO_POINTER (level));
+    }
+
+}
+
 static void
 layout_manager_real_get_preferred_width (ClutterLayoutManager *manager,
                                          ClutterContainer     *container,
@@ -661,9 +713,17 @@ clutter_layout_manager_allocate (ClutterLayoutManager   *manager,
 void
 clutter_layout_manager_layout_changed (ClutterLayoutManager *manager)
 {
+  gpointer is_frozen;
+
   g_return_if_fail (CLUTTER_IS_LAYOUT_MANAGER (manager));
 
-  g_signal_emit (manager, manager_signals[LAYOUT_CHANGED], 0);
+  is_frozen = g_object_get_data (G_OBJECT (manager), "freeze-change");
+  if (is_frozen == NULL)
+    g_signal_emit (manager, manager_signals[LAYOUT_CHANGED], 0);
+  else
+    CLUTTER_NOTE (LAYOUT, "Layout manager '%s'[%p] has been frozen",
+                  G_OBJECT_TYPE_NAME (manager),
+                  manager);
 }
 
 /**
@@ -706,12 +766,17 @@ create_child_meta (ClutterLayoutManager *manager,
                    ClutterActor         *actor)
 {
   ClutterLayoutManagerClass *klass;
+  ClutterLayoutMeta *meta = NULL;
+
+  layout_manager_freeze_layout_change (manager);
 
   klass = CLUTTER_LAYOUT_MANAGER_GET_CLASS (manager);
   if (klass->get_child_meta_type (manager) != G_TYPE_INVALID)
-    return klass->create_child_meta (manager, container, actor);
+    meta = klass->create_child_meta (manager, container, actor);
 
-  return NULL;
+  layout_manager_thaw_layout_change (manager);
+
+  return meta;
 }
 
 static inline ClutterLayoutMeta *
@@ -760,8 +825,10 @@ get_child_meta (ClutterLayoutManager *manager,
  * to the @actor child of @container, eventually by creating one if the
  * #ClutterLayoutManager supports layout properties
  *
- * Return value: a #ClutterLayoutMeta, or %NULL if the #ClutterLayoutManager
- *   does not have layout properties
+ * Return value: (transfer none): a #ClutterLayoutMeta, or %NULL if the
+ *   #ClutterLayoutManager does not have layout properties. The returned
+ *   layout meta instance is owned by the #ClutterLayoutManager and it
+ *   should not be unreferenced
  *
  * Since: 1.0
  */
@@ -886,14 +953,9 @@ clutter_layout_manager_child_set (ClutterLayoutManager *manager,
           break;
         }
 
-#if GLIB_CHECK_VERSION (2, 23, 2)
       G_VALUE_COLLECT_INIT (&value, G_PARAM_SPEC_VALUE_TYPE (pspec),
                             var_args, 0,
                             &error);
-#else
-      g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (pspec));
-      G_VALUE_COLLECT (&value, var_args, 0, &error);
-#endif /* GLIB_CHECK_VERSION (2, 23, 2) */
 
       if (error)
         {
@@ -1163,9 +1225,9 @@ clutter_layout_manager_find_child_property (ClutterLayoutManager *manager,
  * Retrieves all the #GParamSpec<!-- -->s for the layout properties
  * stored inside the #ClutterLayoutMeta sub-class used by @manager
  *
- * Return value: (transfer full): the newly-allocated, %NULL-terminated
- *   array of #GParamSpec<!-- -->s. Use g_free() to free the resources
- *   allocated for the array
+ * Return value: (transfer full) (array length=n_pspecs): the newly-allocated,
+ *   %NULL-terminated array of #GParamSpec<!-- -->s. Use g_free() to free the
+ *   resources allocated for the array
  *
  * Since: 1.2
  */
