@@ -45,17 +45,18 @@
 #endif
 
 #include "clutter-texture.h"
+
+#include "clutter-actor-private.h"
+#include "clutter-debug.h"
+#include "clutter-enum-types.h"
+#include "clutter-feature.h"
 #include "clutter-main.h"
 #include "clutter-marshal.h"
-#include "clutter-feature.h"
-#include "clutter-util.h"
 #include "clutter-private.h"
 #include "clutter-scriptable.h"
-#include "clutter-debug.h"
-#include "clutter-fixed.h"
-#include "clutter-enum-types.h"
-
-#include "cogl/cogl.h"
+#include "clutter-shader.h"
+#include "clutter-stage-private.h"
+#include "clutter-util.h"
 
 static void clutter_scriptable_iface_init (ClutterScriptableIface *iface);
 
@@ -990,7 +991,8 @@ clutter_texture_class_init (ClutterTextureClass *klass)
 
   pspec = g_param_spec_boolean ("disable-slicing",
                                 P_("Disable Slicing"),
-                                P_("Forces the underlying texture to be singlular and not made of of smaller space saving inidivual textures."),
+                                P_("Forces the underlying texture to be singular and not made of smaller space saving "
+                                   "individual textures"),
                                 FALSE,
                                 G_PARAM_CONSTRUCT_ONLY |
                                 CLUTTER_PARAM_READWRITE);
@@ -1008,7 +1010,7 @@ clutter_texture_class_init (ClutterTextureClass *klass)
 
   pspec = g_param_spec_boolean ("repeat-x",
                                 P_("Horizontal repeat"),
-                                P_("Repeat the contents rather than scaling them horizontally."),
+                                P_("Repeat the contents rather than scaling them horizontally"),
                                 FALSE,
                                 CLUTTER_PARAM_READWRITE);
   obj_props[PROP_REPEAT_X] = pspec;
@@ -1016,7 +1018,7 @@ clutter_texture_class_init (ClutterTextureClass *klass)
 
   pspec = g_param_spec_boolean ("repeat-y",
                                 P_("Vertical repeat"),
-                                P_("Repeat the contents rather than scaling them vertically."),
+                                P_("Repeat the contents rather than scaling them vertically"),
                                 FALSE,
                                 CLUTTER_PARAM_READWRITE);
   obj_props[PROP_REPEAT_Y] = pspec;
@@ -1024,7 +1026,7 @@ clutter_texture_class_init (ClutterTextureClass *klass)
 
   pspec = g_param_spec_enum ("filter-quality",
                              P_("Filter Quality"),
-                             P_("Rendering quality used when drawing the texture."),
+                             P_("Rendering quality used when drawing the texture"),
                              CLUTTER_TYPE_TEXTURE_QUALITY,
                              CLUTTER_TEXTURE_QUALITY_MEDIUM,
                              G_PARAM_CONSTRUCT | CLUTTER_PARAM_READWRITE);
@@ -1033,7 +1035,7 @@ clutter_texture_class_init (ClutterTextureClass *klass)
 
   pspec = g_param_spec_enum ("pixel-format",
                              P_("Pixel Format"),
-                             P_("The Cogl pixel format to use."),
+                             P_("The Cogl pixel format to use"),
                              COGL_TYPE_PIXEL_FORMAT,
                              COGL_PIXEL_FORMAT_RGBA_8888,
                              CLUTTER_PARAM_READABLE);
@@ -1041,16 +1043,16 @@ clutter_texture_class_init (ClutterTextureClass *klass)
   g_object_class_install_property (gobject_class, PROP_PIXEL_FORMAT, pspec);
 
   pspec = g_param_spec_boxed ("cogl-texture",
-                              P_("COGL Texture"),
-                              P_("The underlying COGL texture handle used to draw this actor"),
+                              P_("Cogl Texture"),
+                              P_("The underlying Cogl texture handle used to draw this actor"),
                               COGL_TYPE_HANDLE,
                               CLUTTER_PARAM_READWRITE);
   obj_props[PROP_COGL_TEXTURE] = pspec;
   g_object_class_install_property (gobject_class, PROP_COGL_TEXTURE, pspec);
 
   pspec = g_param_spec_boxed ("cogl-material",
-                              P_("COGL Material"),
-                              P_("The underlying COGL material handle used to draw this actor"),
+                              P_("Cogl Material"),
+                              P_("The underlying Cogl material handle used to draw this actor"),
                               COGL_TYPE_HANDLE,
                               CLUTTER_PARAM_READWRITE);
   obj_props[PROP_COGL_MATERIAL] = pspec;
@@ -1102,7 +1104,7 @@ clutter_texture_class_init (ClutterTextureClass *klass)
    */
   pspec = g_param_spec_boolean ("load-async",
                                 P_("Load asynchronously"),
-                                P_("Load files inside a thread to avoid blocking when loading images from disk."),
+                                P_("Load files inside a thread to avoid blocking when loading images from disk"),
                                 FALSE,
                                 CLUTTER_PARAM_WRITABLE);
   obj_props[PROP_LOAD_ASYNC] = pspec;
@@ -1119,7 +1121,7 @@ clutter_texture_class_init (ClutterTextureClass *klass)
    */
   pspec = g_param_spec_boolean ("load-data-async",
                                 P_("Load data asynchronously"),
-                                P_("Decode image data files inside a thread to reduce blocking when loading images from disk."),
+                                P_("Decode image data files inside a thread to reduce blocking when loading images from disk"),
                                 FALSE,
                                 CLUTTER_PARAM_WRITABLE);
   obj_props[PROP_LOAD_DATA_ASYNC] = pspec;
@@ -1227,7 +1229,10 @@ clutter_texture_set_custom_property (ClutterScriptable *scriptable,
 
       path = clutter_script_lookup_filename (script, str);
       if (G_UNLIKELY (!path))
-        return;
+        {
+          g_warning ("Unable to find image %s", str);
+          return;
+        }
 
       error = NULL;
       clutter_texture_set_from_file (texture, path, &error);
@@ -1539,7 +1544,7 @@ clutter_texture_set_from_data (ClutterTexture     *texture,
 
       g_set_error (&inner_error, CLUTTER_TEXTURE_ERROR,
                    CLUTTER_TEXTURE_ERROR_BAD_FORMAT,
-                   "Failed to create COGL texture");
+                   "Failed to create Cogl texture");
 
       g_signal_emit (texture, texture_signals[LOAD_FINISHED], 0, inner_error);
 
@@ -1935,7 +1940,7 @@ clutter_texture_async_load (ClutterTexture *self,
     {
       g_set_error (error, CLUTTER_TEXTURE_ERROR,
 		   CLUTTER_TEXTURE_ERROR_BAD_FORMAT,
-		   "Failed to create COGL texture");
+		   "Failed to create Cogl texture");
       return FALSE;
     }
   else
@@ -2030,7 +2035,7 @@ clutter_texture_set_from_file (ClutterTexture *texture,
     {
       g_set_error (&internal_error, CLUTTER_TEXTURE_ERROR,
                    CLUTTER_TEXTURE_ERROR_BAD_FORMAT,
-		   "Failed to create COGL texture");
+		   "Failed to create Cogl texture");
     }
 
   if (internal_error != NULL)
@@ -2337,7 +2342,7 @@ clutter_texture_set_area_from_rgb_data (ClutterTexture     *texture,
     {
       g_set_error (error, CLUTTER_TEXTURE_ERROR,
 		   CLUTTER_TEXTURE_ERROR_BAD_FORMAT,
-		   "Failed to upload COGL texture data");
+		   "Failed to upload Cogl texture data");
       return FALSE;
     }
 
@@ -2514,8 +2519,8 @@ fbo_source_queue_relayout_cb (ClutterActor *source,
  *   </listitem>
  *   <listitem>
  *     <para>Uploading pixel data to the texture (e.g by using
- *     clutter_actor_set_from_file()) will destroy the offscreen texture data
- *     and end redirection.</para>
+ *     clutter_texture_set_from_file()) will destroy the offscreen texture
+ *     data and end redirection.</para>
  *   </listitem>
  *   <listitem>
  *     <para>cogl_texture_get_data() with the handle returned by
