@@ -42,107 +42,14 @@
 #include "cogl-matrix-private.h"
 #include "cogl-primitives-private.h"
 
-typedef struct _CoglClipStackRect CoglClipStackRect;
-typedef struct _CoglClipStackWindowRect CoglClipStackWindowRect;
-typedef struct _CoglClipStackPath CoglClipStackPath;
-
-typedef enum
-  {
-    COGL_CLIP_STACK_RECT,
-    COGL_CLIP_STACK_WINDOW_RECT,
-    COGL_CLIP_STACK_PATH
-  } CoglClipStackType;
-
-/* A clip stack consists a list of entries. Each entry has a reference
- * count and a link to its parent node. The child takes a reference on
- * the parent and the CoglClipStack holds a reference to the top of
- * the stack. There are no links back from the parent to the
- * children. This allows stacks that have common ancestry to share the
- * entries.
- *
- * For example, the following sequence of operations would generate
- * the tree below:
- *
- * CoglClipStack *stack_a = NULL;
- * stack_a = _cogl_clip_stack_push_rectangle (stack_a, ...);
- * stack_a = _cogl_clip_stack_push_rectangle (stack_a, ...);
- * stack_a = _cogl_clip_stack_push_from_path (stack_a, ...);
- * CoglClipStack *stack_b = NULL;
- * stack_b = cogl_clip_stack_push_window_rectangle (stack_b, ...);
- *
- *  stack_a
- *         \ holds a ref to
- *          +-----------+
- *          | path node |
- *          |ref count 1|
- *          +-----------+
- *                       \
- *                        +-----------+  +-----------+
- *       both tops hold   | rect node |  | rect node |
- *       a ref to the     |ref count 2|--|ref count 1|
- *       same rect node   +-----------+  +-----------+
- *                       /
- *          +-----------+
- *          | win. rect |
- *          |ref count 1|
- *          +-----------+
- *         / holds a ref to
- *  stack_b
- *
- */
-
-struct _CoglClipStack
-{
-  CoglClipStackType  type;
-
-  /* This will be null if there is no parent. If it is not null then
-     this node must be holding a reference to the parent */
-  CoglClipStack     *parent;
-
-  /* All clip entries have a window-space bounding box which we can
-     use to calculate a scissor. The scissor limits the clip so that
-     we don't need to do a full stencil clear if the stencil buffer is
-     needed. This is stored in Cogl's coordinate space (ie, 0,0 is the
-     top left) */
-  int                     bounds_x0;
-  int                     bounds_y0;
-  int                     bounds_x1;
-  int                     bounds_y1;
-
-  unsigned int            ref_count;
-};
-
-struct _CoglClipStackRect
-{
-  CoglClipStack     _parent_data;
-
-  /* The rectangle for this clip */
-  float                  x0;
-  float                  y0;
-  float                  x1;
-  float                  y1;
-
-  /* The matrix that was current when the clip was set */
-  CoglMatrix             matrix;
-};
-
-struct _CoglClipStackWindowRect
-{
-  CoglClipStack     _parent_data;
-
-  /* The window rect clip doesn't need any specific data because it
-     just adds to the scissor clip */
-};
-
-struct _CoglClipStackPath
-{
-  CoglClipStack     _parent_data;
-
-  /* The matrix that was current when the clip was set */
-  CoglMatrix             matrix;
-
-  CoglPath              *path;
-};
+#ifndef GL_CLIP_PLANE0
+#define GL_CLIP_PLANE0 0x3000
+#define GL_CLIP_PLANE1 0x3001
+#define GL_CLIP_PLANE2 0x3002
+#define GL_CLIP_PLANE3 0x3003
+#define GL_CLIP_PLANE4 0x3004
+#define GL_CLIP_PLANE5 0x3005
+#endif
 
 static void
 project_vertex (const CoglMatrix *modelview_projection,
@@ -207,7 +114,9 @@ set_clip_plane (GLint plane_num,
   plane[1] = -1.0;
   plane[2] = 0;
   plane[3] = vertex_a[1];
-#if defined (HAVE_COGL_GLES2) || defined (HAVE_COGL_GLES)
+#if defined (HAVE_COGL_GLES2)
+  g_assert_not_reached ();
+#elif defined (HAVE_COGL_GLES)
   GE( glClipPlanef (plane_num, plane) );
 #else
   GE( glClipPlane (plane_num, plane) );
@@ -470,6 +379,8 @@ _cogl_clip_stack_push_rectangle (CoglClipStack *stack,
                                  const CoglMatrix *modelview_matrix)
 {
   CoglClipStackRect *entry;
+  CoglMatrix matrix_p;
+  float v[4];
 
   /* Make a new entry */
   entry = _cogl_clip_stack_push_entry (stack,
@@ -483,8 +394,48 @@ _cogl_clip_stack_push_rectangle (CoglClipStack *stack,
 
   entry->matrix = *modelview_matrix;
 
-  _cogl_clip_stack_entry_set_bounds ((CoglClipStack *) entry,
-                                     x_1, y_1, x_2, y_2, modelview_matrix);
+  /* If the modelview meets these constraints then a transformed rectangle
+   * should still be a rectangle when it reaches screen coordinates.
+   *
+   * FIXME: we are are making certain assumptions about the projection
+   * matrix a.t.m and should really be looking at the combined modelview
+   * and projection matrix.
+   * FIXME: we don't consider rotations that are a multiple of 90 degrees
+   * which could be quite common.
+   */
+  if (modelview_matrix->xy != 0 || modelview_matrix->xz != 0 ||
+      modelview_matrix->yx != 0 || modelview_matrix->yz != 0 ||
+      modelview_matrix->zx != 0 || modelview_matrix->zy != 0)
+    {
+      entry->can_be_scissor = FALSE;
+      _cogl_clip_stack_entry_set_bounds ((CoglClipStack *) entry,
+                                         x_1, y_1, x_2, y_2, modelview_matrix);
+    }
+  else
+    {
+      CoglClipStack *base_entry = (CoglClipStack *) entry;
+
+      cogl_get_projection_matrix (&matrix_p);
+      cogl_get_viewport (v);
+
+      _cogl_transform_point (modelview_matrix, &matrix_p, v, &x_1, &y_1);
+      _cogl_transform_point (modelview_matrix, &matrix_p, v, &x_2, &y_2);
+
+      /* Consider that the modelview matrix may flip the rectangle
+       * along the x or y axis... */
+#define SWAP(A,B) do { float tmp = B; B = A; A = tmp; } while (0)
+      if (x_1 > x_2)
+        SWAP (x_1, x_2);
+      if (y_1 > y_2)
+        SWAP (y_1, y_2);
+#undef SWAP
+
+      base_entry->bounds_x0 = COGL_UTIL_NEARBYINT (x_1);
+      base_entry->bounds_y0 = COGL_UTIL_NEARBYINT (y_1);
+      base_entry->bounds_x1 = COGL_UTIL_NEARBYINT (x_2);
+      base_entry->bounds_y1 = COGL_UTIL_NEARBYINT (y_2);
+      entry->can_be_scissor = TRUE;
+    }
 
   return (CoglClipStack *) entry;
 }
@@ -580,16 +531,41 @@ _cogl_clip_stack_pop (CoglClipStack *stack)
   return new_top;
 }
 
+void
+_cogl_clip_stack_get_bounds (CoglClipStack *stack,
+                             int *scissor_x0,
+                             int *scissor_y0,
+                             int *scissor_x1,
+                             int *scissor_y1)
+{
+  CoglClipStack *entry;
+
+  *scissor_x0 = 0;
+  *scissor_y0 = 0;
+  *scissor_x1 = G_MAXINT;
+  *scissor_y1 = G_MAXINT;
+
+  for (entry = stack; entry; entry = entry->parent)
+    {
+      /* Get the intersection of the current scissor and the bounding
+         box of this clip */
+      *scissor_x0 = MAX (*scissor_x0, entry->bounds_x0);
+      *scissor_y0 = MAX (*scissor_y0, entry->bounds_y0);
+      *scissor_x1 = MIN (*scissor_x1, entry->bounds_x1);
+      *scissor_y1 = MIN (*scissor_y1, entry->bounds_y1);
+    }
+}
+
 void
 _cogl_clip_stack_flush (CoglClipStack *stack)
 {
   int has_clip_planes;
   gboolean using_clip_planes = FALSE;
   gboolean using_stencil_buffer = FALSE;
-  int scissor_x0 = 0;
-  int scissor_y0 = 0;
-  int scissor_x1 = G_MAXINT;
-  int scissor_y1 = G_MAXINT;
+  int scissor_x0;
+  int scissor_y0;
+  int scissor_x1;
+  int scissor_y1;
   CoglMatrixStack *modelview_stack;
   CoglClipStack *entry;
   int scissor_y_start;
@@ -630,15 +606,9 @@ _cogl_clip_stack_flush (CoglClipStack *stack)
      clear the stencil buffer then the clear will be clipped to the
      intersection of all of the bounding boxes. This saves having to
      clear the whole stencil buffer */
-  for (entry = stack; entry; entry = entry->parent)
-    {
-      /* Get the intersection of the current scissor and the bounding
-         box of this clip */
-      scissor_x0 = MAX (scissor_x0, entry->bounds_x0);
-      scissor_y0 = MAX (scissor_y0, entry->bounds_y0);
-      scissor_x1 = MIN (scissor_x1, entry->bounds_x1);
-      scissor_y1 = MIN (scissor_y1, entry->bounds_y1);
-    }
+  _cogl_clip_stack_get_bounds (stack,
+                               &scissor_x0, &scissor_y0,
+                               &scissor_x1, &scissor_y1);
 
   /* Enable scissoring as soon as possible */
   if (scissor_x0 >= scissor_x1 || scissor_y0 >= scissor_y1)
@@ -696,32 +666,37 @@ _cogl_clip_stack_flush (CoglClipStack *stack)
         {
           CoglClipStackRect *rect = (CoglClipStackRect *) entry;
 
-          _cogl_matrix_stack_push (modelview_stack);
-          _cogl_matrix_stack_set (modelview_stack, &rect->matrix);
-
-          /* If we support clip planes and we haven't already used
-             them then use that instead */
-          if (has_clip_planes)
+          /* We don't need to do anything extra if the clip for this
+             rectangle was entirely described by its scissor bounds */
+          if (!rect->can_be_scissor)
             {
-              set_clip_planes (rect->x0,
-                               rect->y0,
-                               rect->x1,
-                               rect->y1);
-              using_clip_planes = TRUE;
-              /* We can't use clip planes a second time */
-              has_clip_planes = FALSE;
-            }
-          else
-            {
-              add_stencil_clip_rectangle (rect->x0,
-                                          rect->y0,
-                                          rect->x1,
-                                          rect->y1,
-                                          !using_stencil_buffer);
-              using_stencil_buffer = TRUE;
-            }
+              _cogl_matrix_stack_push (modelview_stack);
+              _cogl_matrix_stack_set (modelview_stack, &rect->matrix);
 
-          _cogl_matrix_stack_pop (modelview_stack);
+              /* If we support clip planes and we haven't already used
+                 them then use that instead */
+              if (has_clip_planes)
+                {
+                  set_clip_planes (rect->x0,
+                                   rect->y0,
+                                   rect->x1,
+                                   rect->y1);
+                  using_clip_planes = TRUE;
+                  /* We can't use clip planes a second time */
+                  has_clip_planes = FALSE;
+                }
+              else
+                {
+                  add_stencil_clip_rectangle (rect->x0,
+                                              rect->y0,
+                                              rect->x1,
+                                              rect->y1,
+                                              !using_stencil_buffer);
+                  using_stencil_buffer = TRUE;
+                }
+
+              _cogl_matrix_stack_pop (modelview_stack);
+            }
         }
       /* We don't need to do anything for window space rectangles
          because their functionality is entirely implemented by the
