@@ -525,8 +525,7 @@ _cogl_pipeline_progend_glsl_end (CoglPipeline *pipeline,
           COGL_PIPELINE_STATE_AFFECTS_FRAGMENT_CODEGEN) &
          ~COGL_PIPELINE_STATE_LAYERS,
          COGL_PIPELINE_LAYER_STATE_AFFECTS_FRAGMENT_CODEGEN |
-         COGL_PIPELINE_LAYER_STATE_AFFECTS_VERTEX_CODEGEN,
-         COGL_PIPELINE_FIND_EQUIVALENT_COMPARE_TEXTURE_TARGET);
+         COGL_PIPELINE_LAYER_STATE_AFFECTS_VERTEX_CODEGEN);
 
       priv = get_glsl_priv (authority);
 
